@@ -1901,7 +1901,7 @@ int main(int argc, char *argv[])
 			 * caller gets signals for its call and not the possible signals
 			 * sent by status_changed()
 			 */
-			check_dbus_messages(channels, MAXDEVICES);
+			check_dbus_messages(channel, MAXDEVICES);
 
 			if (changed) {
 				/* Send status message when something was changed */
@@ -1925,11 +1925,12 @@ int main(int argc, char *argv[])
 						last_report_status = 1; /* not empty */
 						gpsd_report(3,
 							    "GPS last report saved, "
-							    "lat=%f(%d), lon=%f, "
-							    "mode=%d(%d)\n",
+							    "lat=%f(%d), lon=%f, alt=%f(%d) mode=%d(%d)\n",
 							    gpsdata->fix.latitude,
 							    (changed & LATLON_SET) == LATLON_SET,
-							    gpsdata->fix.longitude, 
+							    gpsdata->fix.longitude,
+							    gpsdata->fix.altitude,
+							    (changed & ALTITUDE_SET) == ALTITUDE_SET,
 							    gpsdata->fix.mode,
 							    ((changed & MODE_SET) == MODE_SET));
 					}
@@ -2099,6 +2100,70 @@ OUT:
 /*@ +mustfreefresh @*/
 
 
+//#define DEBUG_LAST_REPORT
+#ifdef DEBUG_LAST_REPORT
+static void
+dump_report (struct gps_data_t *gps_data)
+{
+	printf("Dumping last saved report\n");
+	printf("Fields set - %d (0x%x)\n", gps_data->set, gps_data->set);
+	printf("Time set - %f\n", gps_data->fix.time);
+	printf("Latitude - %f\n", gps_data->fix.latitude);
+	printf("Longitude - %f\n", gps_data->fix.longitude);
+	printf("Altitude - %f\n", gps_data->fix.altitude);
+	printf("Track - %f\n", gps_data->fix.track);
+	printf("Speed - %f\n", gps_data->fix.speed);
+	printf("Climb - %f\n", gps_data->fix.climb);
+
+	printf("Satellites in view - %d\n", gps_data->satellites);
+	printf("Satellites in use - %d\n", gps_data->satellites_used);
+}
+
+
+#define DUMP(flag)				\
+	if ((set) & (flag))			\
+		printf("%s ", #flag)
+
+void dump_set(gps_mask_t set)
+{
+	DUMP(ONLINE_SET);
+	DUMP(TIME_SET);
+	DUMP(TIMERR_SET);
+	DUMP(LATLON_SET);
+	DUMP(ALTITUDE_SET);
+	DUMP(SPEED_SET);
+	DUMP(TRACK_SET);
+	DUMP(CLIMB_SET);
+	DUMP(STATUS_SET);
+	DUMP(MODE_SET);
+	DUMP(HDOP_SET);
+	DUMP(VDOP_SET);
+	DUMP(PDOP_SET);
+	DUMP(TDOP_SET);
+	DUMP(GDOP_SET);
+	DUMP(DOP_SET);
+	DUMP(HERR_SET);
+	DUMP(VERR_SET);
+	DUMP(PERR_SET);
+	DUMP(SATELLITE_SET);
+	DUMP(PSEUDORANGE_SET);
+	DUMP(USED_SET);
+	DUMP(SPEEDERR_SET);
+	DUMP(TRACKERR_SET);
+	DUMP(CLIMBERR_SET);
+	DUMP(DEVICE_SET);
+	DUMP(DEVICELIST_SET);
+	DUMP(DEVICEID_SET);
+	DUMP(ERROR_SET);
+	DUMP(CYCLE_START_SET);
+	DUMP(RTCM_SET);
+	DUMP(FIX_SET);
+
+	printf("\n");
+}
+#endif
+
+
 /* This is only called from gpsd.
  * Returns:
  *   <0 : error happened, report was not saved
@@ -2123,6 +2188,12 @@ static int gps_save_last_report(void)
 		}
 		close(fd);
 		st = 1;
+
+#ifdef DEBUG_LAST_REPORT
+		printf("Current = "); dump_set(last_report.gpsdata.set);
+		printf("Accumulated = "); dump_set(last_report.set);
+		dump_report(&last_report.gpsdata);
+#endif
 	}
 
 	return st;
