@@ -12,6 +12,8 @@
 #include <pthread.h>	/* pacifies OpenBSD's compiler */
 #endif
 #include <math.h>
+#include <netinet/in.h>
+#include <netinet/tcp.h>
 
 #include "gpsd.h"
 
@@ -129,6 +131,9 @@ struct gps_data_t *gps_open(const char *host, const char *port)
 /* open a connection to a gpsd daemon */
 {
     struct gps_data_t *gpsdata = (struct gps_data_t *)calloc(sizeof(struct gps_data_t), 1);
+#ifndef USE_DOMAIN_SOCKET_IPC
+    int optval = 1;
+#endif
 
     /*@ -branchstate @*/
     if (!gpsdata)
@@ -138,12 +143,34 @@ struct gps_data_t *gps_open(const char *host, const char *port)
     if (!port)
 	port = DEFAULT_GPSD_PORT;
 
+#ifdef USE_DOMAIN_SOCKET_IPC
+    if ((gpsdata->gps_fd = netlib_connectsock_domain(GPSD_SERVER_CLIENT_PATH))<0) {
+	errno = gpsdata->gps_fd;
+	(void)free(gpsdata);
+	return NULL;
+    }
+#else
     if ((gpsdata->gps_fd = netlib_connectsock(host, port, "tcp")) < 0) {
 	errno = gpsdata->gps_fd;
 	(void)free(gpsdata);
 	return NULL;
     }
 
+#ifdef DISABLE_NAGLE
+    /* No Nagle, it just delays things unnecessarily */
+    if (setsockopt(gpsdata->gps_fd,
+		   IPPROTO_TCP,
+		   TCP_NODELAY,
+		   (const char *)&optval,
+		   sizeof(int))<0) {
+	    printf("setsockopt(TCP_NODELAY, #%d) failed (%d/%s)",
+		   gpsdata->gps_fd,
+		   errno,
+		   strerror(errno));
+    }
+#endif
+#endif
+
     gpsdata->status = STATUS_NO_FIX;
     gps_clear_fix(&gpsdata->fix);
     return gpsdata;
@@ -186,10 +213,10 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 
     for (ns = buf; ns; ns = strstr(ns+1, "GPSD")) {
 	if (/*@i1@*/strncmp(ns, "GPSD", 4) == 0) {
-	    for (sp = ns + 5; ; sp = tp) {
+	    for (sp = ns + 5; *sp != '\0'; sp = tp+1) {
 		tp = sp + strcspn(sp, ",\r\n");
-		if (*tp == '\0') break;
-		*tp = '\0';
+		if (*tp == '\0') tp--;
+		else *tp = '\0';
 
 		switch (*sp) {
 		case 'A':
@@ -250,11 +277,11 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 		    break;
 		case 'I':
 		    /*@ -mustfreeonly */
+		    if (gpsdata->gps_id)
+			free(gpsdata->gps_id);
 		    if (sp[2] == '?') 
 			gpsdata->gps_id = NULL;
 		    else {
-			if (gpsdata->gps_id)
-			    free(gpsdata->gps_id);
 			gpsdata->gps_id = strdup(sp+2);
 			gpsdata->set |= DEVICEID_SET;
 		    }
@@ -276,7 +303,7 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 			    (size_t)gpsdata->ndevices,
 			    sizeof(char **));
 			/*@ -nullstate @*/
-			gpsdata->devicelist[i=0] = strtok_r(sp+2, " \r\n", &ns);
+			gpsdata->devicelist[i=0] = strdup(strtok_r(sp+2, " \r\n", &ns));
 			while ((sp = strtok_r(NULL, " \r\n",  &ns)))
 			    gpsdata->devicelist[++i] = strdup(sp);
 			/*@ +nullstate @*/
@@ -425,9 +452,9 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 			int PRN[MAXCHANNELS];
 			int elevation[MAXCHANNELS], azimuth[MAXCHANNELS];
 			int ss[MAXCHANNELS], used[MAXCHANNELS];
-			char tag[21], timestamp[21];
+			char tag[MAXTAGLEN+1], timestamp[21];
 
-			(void)sscanf(sp, "Y=%20s %20s %d ", 
+			(void)sscanf(sp, "Y=%8s %20s %d ", 
 			       tag, timestamp, &gpsdata->satellites);
 			(void)strncpy(gpsdata->tag, tag, MAXTAGLEN);
 			if (timestamp[0] != '?') {
@@ -459,16 +486,32 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 		    break;
 		case '$':
 		    /*@ +matchanyintegral @*/
-		    (void)sscanf(sp, "$=%s %ld %lf %lf %lf %lf %lf %lf", 
+		    if (!gpsdata->profiling)
+			    break;
+		    (void)sscanf(sp, "$=%8s %ld %lf %lf %lf %lf %lf %lf", 
 			   gpsdata->tag,
 			   (long *)&gpsdata->sentence_length,
-			   &gpsdata->fix.time, 
+			   &gpsdata->sentence_time, 
 			   &gpsdata->d_xmit_time, 
 			   &gpsdata->d_recv_time, 
 			   &gpsdata->d_decode_time, 
 			   &gpsdata->poll_time, 
 			   &gpsdata->emit_time);
 		    /*@ -matchanyintegral @*/
+#ifdef DEBUG_PROFILE
+#if 0
+		    //fprintf(stderr, "profile receive: \"%s\"\n", sp);
+		    printf("profile receive: tag=%s, len=%ld, sentence_time=%f, d_xmit_time=%f, d_recv_time=%f, d_decode_time=%f, poll=%f, emit=%f\n",
+			   gpsdata->tag,
+			   (long)gpsdata->sentence_length,
+			   gpsdata->sentence_time, 
+			   gpsdata->d_xmit_time, 
+			   gpsdata->d_recv_time, 
+			   gpsdata->d_decode_time, 
+			   gpsdata->poll_time, 
+			   gpsdata->emit_time);
+#endif
+#endif
 		    break;
 		}
 	    }
@@ -476,10 +519,6 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
     }
 
 /*@ -nullstate -compdef @*/
-    if (gpsdata->raw_hook)
-	gpsdata->raw_hook(gpsdata, buf, strlen(buf),  1);
-    if (gpsdata->thread_hook)
-	gpsdata->thread_hook(gpsdata, buf, strlen(buf), 1);
 }
 /*@ +nullstate +compdef @*/
 /*@ -branchstate +usereleased @*/
@@ -508,11 +547,78 @@ int gps_poll(struct gps_data_t *gpsdata)
     received = gpsdata->online = timestamp();
     gps_unpack(buf, gpsdata);
 
-    if (gpsdata->profiling)
-    {
-	gpsdata->c_decode_time = received - gpsdata->fix.time;
-	gpsdata->c_recv_time = timestamp() - gpsdata->fix.time;
+    if (gpsdata->profiling) {
+	    if (!isnan(gpsdata->sentence_time) && gpsdata->sentence_time != 0.0) {
+		    gpsdata->c_decode_time = received - gpsdata->sentence_time;
+		    gpsdata->c_recv_time = timestamp() - gpsdata->sentence_time;
+
+#ifdef DEBUG_PROFILE
+		    {
+			    double current_time = timestamp();
+			    static double sum_ddelay = 0.0, sum_tdelay = 0.0,
+				    sum_cdelay = 0.0;
+			    static unsigned long count_delays = 0;
+			    double ddelay, tdelay, cdelay;
+			    double avg_ddelay, avg_tdelay, avg_cdelay;
+
+			    ddelay = gpsdata->emit_time - gpsdata->d_xmit_time;
+			    tdelay = gpsdata->c_decode_time - gpsdata->emit_time;
+			    cdelay = gpsdata->c_recv_time - gpsdata->c_decode_time;
+
+			    count_delays++;
+			    sum_ddelay += ddelay;
+			    sum_tdelay += tdelay;
+			    sum_cdelay += cdelay;
+
+			    avg_ddelay = sum_ddelay / (double)count_delays;
+			    avg_tdelay = sum_tdelay / (double)count_delays;
+			    avg_cdelay = sum_cdelay / (double)count_delays;
+
+			    //fprintf(stderr, "profile receive: \"%s\"\n", sp);
+			    printf("profile(%f) : tag=%s, len=%ld, sentence_time=%f, "
+				   "d_xmit_time=%f, d_recv_time=%f, d_decode_time=%f, "
+				   "poll=%f, emit=%f, c_decode_time=%f, c_recv_time=%f, "
+				   "ddelay=%f, tdelay=%f, cdelay=%f, "
+				   "avg_ddelay=%f, avg_tdelay=%f, avg_cdelay=%f, "
+				   "total_delay=%f, timediff=%f\n"
+				   ,
+				   current_time,
+				   gpsdata->tag,
+				   (long)gpsdata->sentence_length,
+				   gpsdata->sentence_time, 
+				   gpsdata->d_xmit_time, 
+				   gpsdata->d_recv_time, 
+				   gpsdata->d_decode_time, 
+				   gpsdata->poll_time, 
+				   gpsdata->emit_time,
+				   gpsdata->c_decode_time,
+				   gpsdata->c_recv_time,
+				   ddelay, tdelay, cdelay,
+				   avg_ddelay, avg_tdelay, avg_cdelay,
+				   avg_ddelay+avg_tdelay+avg_cdelay,
+				   current_time-gpsdata->sentence_time);
+		    }
+#endif
+	    } else {
+		    /* Just ignore the values because sentence time is not correctly set */
+		    gpsdata->sentence_time = 
+		    gpsdata->d_xmit_time = 
+		    gpsdata->d_recv_time = 
+		    gpsdata->d_decode_time =  
+		    gpsdata->poll_time =
+		    gpsdata->emit_time =
+		    gpsdata->c_decode_time =
+		    gpsdata->c_recv_time =
+		    gpsdata->c_decode_time = 0.0;
+	    }
     }
+
+    /* Profiling will not work if the hooks are called in gps_unpack() */
+    if (gpsdata->raw_hook)
+	gpsdata->raw_hook(gpsdata, buf, strlen(buf),  1);
+    if (gpsdata->thread_hook)
+	gpsdata->thread_hook(gpsdata, buf, strlen(buf), 1);
+
     return 0;
 }
 
