@@ -473,11 +473,47 @@ static void apply_error_model(struct gps_device_t *session)
     }
 }
 
+
+//#define PROFILE_PROCESSING
+#ifdef PROFILE_PROCESSING
+struct profiling {
+	double sum_processing_time;
+	unsigned long count_processing;
+	double max_processing_time;
+
+	double receive_time;
+	double end_time;
+};
+static struct profiling prof;
+
+#define INIT_PROF() \
+	prof.receive_time=timestamp(); \
+	prof.end_time = 0.0;
+
+#define UPDATE_PROF() do {						\
+		double proc_time;					\
+		prof.end_time=timestamp();				\
+		proc_time = prof.end_time - prof.receive_time;		\
+		prof.sum_processing_time += proc_time;			\
+		prof.count_processing++;				\
+		if (prof.max_processing_time < proc_time)		\
+			prof.max_processing_time = proc_time;		\
+	} while(0)
+#endif
+
+
 gps_mask_t gpsd_poll(struct gps_device_t *session)
 /* update the stuff in the scoreboard structure */
 {
     ssize_t newdata;
 
+#ifdef PROFILE_PROCESSING
+    static time_t prof_printed = 0;
+    gpsd_report(0, "%f: packet processing starts\n", timestamp());
+
+    INIT_PROF();
+#endif
+
     if (session->inbuflen==0)
 	session->gpsdata.d_xmit_time = timestamp();
 
@@ -560,7 +596,6 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 				      (char *)session->outbuffer,
 				      (size_t)session->outbuflen, 2);
 	/*@ -nullstate @*/
-	session->gpsdata.sentence_time = NAN;
 	session->gpsdata.sentence_length = session->outbuflen;
 	session->gpsdata.d_recv_time = timestamp();
 
@@ -605,11 +640,15 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 	session->gpsdata.d_decode_time = timestamp();
 
 	/* also copy the sentence up to clients in raw mode */
-	if (session->packet_type == NMEA_PACKET)
+	if (session->packet_type == NMEA_PACKET) {
 	    session->gpsdata.raw_hook(&session->gpsdata,
 				      (char *)session->outbuffer,
 				      strlen((char *)session->outbuffer), 1);
-	else {
+
+#ifdef PROFILE_PROCESSING
+//	    gpsd_report(0, "%f: --> %s (%d)\n", session->gpsdata.d_xmit_time, session->outbuffer, strlen(session->outbuffer));
+#endif
+	} else {
 	    char buf2[MAX_PACKET_LENGTH*3+2];
 
 	    buf2[0] = '\0';
@@ -644,6 +683,21 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 
 	dgpsip_report(session);
 
+#ifdef PROFILE_PROCESSING
+	UPDATE_PROF();
+	//if ((prof_printed+5)<time(0)) {
+	if (1) {
+		char sentence[8+1] = {0};
+		unsigned char *p = strchr(session->outbuffer, ',');
+		if (p && ((p - session->outbuffer)<8)) {
+			strncpy(sentence, session->outbuffer+1, (p - session->outbuffer)-1);
+		} else
+			strcpy(sentence, "?");
+		gpsd_report(0, "%f: processing %s ends: %.2fms (%fs). Time avg = %.2fms, max = %.2fms\n", timestamp(), sentence, (prof.end_time - prof.receive_time)*1000.0, prof.end_time - prof.receive_time, (prof.sum_processing_time/(double)prof.count_processing)*1000.0, prof.max_processing_time*1000.0);
+		prof_printed=time(0);
+	}
+#endif
+
 	return session->gpsdata.set;
     }
 }
