@@ -92,6 +92,18 @@ static void merge_hhmmss(char *hhmmss, struct gps_device_t *session)
 
 #undef DD
 
+
+/**************************************************************************
+ *
+ * Compare GPS timestamps for equality.  Depends on the fact that the
+ * timestamp granularity of GPS is 1/100th of a second.  Use this to avoid
+ * naive float comparisons.
+ *
+ **************************************************************************/
+
+#define GPS_TIME_EQUAL(a, b) (fabs((a) - (b)) < 0.01)
+
+
 /**************************************************************************
  *
  * NMEA sentence handling begins here
@@ -138,7 +150,7 @@ static gps_mask_t processGPRMC(int count, char *field[], struct gps_device_t *se
 	    merge_hhmmss(field[1], session);
 	    mask |= TIME_SET;
 	    session->gpsdata.newdata.time = (double)mkgmtime(&session->driver.nmea.date)+session->driver.nmea.subseconds;
-	    if (session->gpsdata.sentence_time != session->gpsdata.newdata.time)
+            if (!GPS_TIME_EQUAL(session->gpsdata.sentence_time, session->gpsdata.newdata.time))
 		mask |= CYCLE_START_SET;
 	    session->gpsdata.sentence_time = session->gpsdata.newdata.time;
 	}
@@ -205,7 +217,7 @@ static gps_mask_t processGPGLL(int count, char *field[], struct gps_device_t *se
 	else {
 	    mask = TIME_SET;
 	    session->gpsdata.newdata.time = (double)mkgmtime(&session->driver.nmea.date)+session->driver.nmea.subseconds;
-	    if (session->gpsdata.sentence_time != session->gpsdata.newdata.time)
+            if (!GPS_TIME_EQUAL(session->gpsdata.sentence_time, session->gpsdata.newdata.time))
 		mask |= CYCLE_START_SET;
 	    session->gpsdata.sentence_time = session->gpsdata.newdata.time;
 	}
@@ -258,6 +270,9 @@ static gps_mask_t processGPGGA(int c UNUSED, char *field[], struct gps_device_t
 	else {
 	    mask |= TIME_SET;
 	    session->gpsdata.newdata.time = (double)mkgmtime(&session->driver.nmea.date)+session->driver.nmea.subseconds;
+	    if (!GPS_TIME_EQUAL(session->gpsdata.sentence_time, session->gpsdata.newdata.time))
+		    mask |= CYCLE_START_SET;
+            session->gpsdata.sentence_time = session->gpsdata.newdata.time;
 	}
 	do_lat_lon(&field[2], &session->gpsdata);
 	mask |= LATLON_SET;
@@ -477,7 +492,7 @@ static gps_mask_t processGPZDA(int c UNUSED, char *field[], struct gps_device_t
     session->driver.nmea.date.tm_mon = atoi(field[3])-1;
     session->driver.nmea.date.tm_mday = atoi(field[2]);
     session->gpsdata.newdata.time = (double)mkgmtime(&session->driver.nmea.date)+session->driver.nmea.subseconds;
-    if (session->gpsdata.sentence_time != session->gpsdata.newdata.time)
+    if (!GPS_TIME_EQUAL(session->gpsdata.sentence_time, session->gpsdata.newdata.time))
 	mask |= CYCLE_START_SET;
     session->gpsdata.sentence_time = session->gpsdata. newdata.time;
     return mask;
