@@ -32,6 +32,9 @@ distinguish them from baud barf.
 #include "config.h"
 #include "gpsd.h"
 
+//#define STATE_DEBUG
+//#define READ_DEBUG
+
 /* 
  * The packet-recognition state machine.  It can be fooled by garbage
  * that looks like the head of a binary packet followed by a NMEA
@@ -61,10 +64,6 @@ distinguish them from baud barf.
  *
  */
 
-enum {
-#include "packet_states.h"
-};
-
 static void nextstate(struct gps_device_t *session, unsigned char c)
 {
 #ifdef RTCM104_ENABLE
@@ -649,6 +648,7 @@ ssize_t packet_parse(struct gps_device_t *session, size_t newdata)
 	    bool checksum_ok = true;
 	    char csum[3];
 	    char *trailer = (char *)session->inbufptr-5;
+	    unsigned char *end = session->inbufptr;
 	    if (*trailer == '*') {
 		unsigned int n, crc = 0;
 		for (n = 1; (char *)session->inbuffer + n < trailer; n++)
@@ -662,6 +662,11 @@ ssize_t packet_parse(struct gps_device_t *session, size_t newdata)
 	    else
 		session->packet_state = GROUND_STATE;
 	    packet_discard(session);
+	    if (newdata==0) {
+		    /* return non-null value so that gps_poll() accepts the packet */
+		    newdata = end - session->inbuffer;
+	    }
+
             break;
 #endif /* NMEA_ENABLE */
 #ifdef SIRFII_ENABLE
@@ -791,9 +796,41 @@ ssize_t packet_get(struct gps_device_t *session)
 /* grab a packet; returns ether BAD_PACKET or the length */
 {
     ssize_t newdata;
+    ssize_t remaining = 0; // how many sentence bytes is left in the buffer
     /*@ -modobserver @*/
-    newdata = read(session->gpsdata.gps_fd, session->inbuffer+session->inbuflen,
-			sizeof(session->inbuffer)-(session->inbuflen));
+
+    remaining = session->inbuflen -
+	    (session->inbufptr - session->inbuffer);
+
+#ifdef READ_DEBUG
+    gpsd_report(0, "Reading data offset %d (left = %d, state = %d, outlen = %d)\n",
+		session->inbufptr - session->inbuffer, remaining,
+		session->packet_state, session->outbuflen);
+#endif /* READ_DEBUG */
+
+    /* Is there data in the buffer that is not read yet, consume remaining
+     * buffer data first before reading socket. Fixes: NB#74463
+     */
+    if (remaining>0 && session->outbuflen>0 && session->packet_state == NMEA_RECOGNIZED) {
+#ifdef READ_DEBUG
+	    gpsd_report(0, "outbuf = \"%s\"\n", session->outbuffer);
+	    gpsd_report(0, "inbuf = \"%s\"\n", session->inbuffer);
+#endif /* READ_DEBUG */
+	    return packet_parse(session, 0);
+    } else {
+	    newdata = read(session->gpsdata.gps_fd, session->inbuffer+session->inbuflen,
+			   sizeof(session->inbuffer)-(session->inbuflen));
+    }
+
+#ifdef READ_DEBUG
+    gpsd_report(0, "Read %d chars to buffer offset %d (total %d): %s\n",
+		newdata,
+		session->inbuflen,
+		session->inbuflen+newdata,
+		//gpsd_hexdump(session->inbufptr, newdata));
+		session->inbufptr);
+#endif /* READ_DEBUG */
+
     /*@ +modobserver @*/
     if (newdata == -1){ /*newdata == -1 && errno = (EAGAIN || EINTR) */
         if ((errno == EAGAIN) || (errno == EINTR)) {
